/*
 * Copyright (C) 2017 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "CommandRunner.h"

QString CommandRunner::getEnvironmentVariable(const QString& variable) const
{
	return qgetenv(variable.toLatin1());
}

bool CommandRunner::isEnvironmentVariableSet(const QString &variable) const
{
	return QProcessEnvironment::systemEnvironment().contains(variable);
}

QString CommandRunner::readFile(const QString &path) const
{
	QFile file(path);
	if (file.open(QFile::ReadOnly | QFile::Text)) {
		QTextStream inputStream(&file);
		return inputStream.readAll();
	} else {
		return {};
	}
}
