#!/bin/bash

echo "MAKE_BINARY=nmake" >> $GITHUB_ENV
echo "CMAKE_GENERATOR=NMake Makefiles" >> $GITHUB_ENV

echo "LIB=$LIB;$INSTALL_PREFIX/lib" >> $GITHUB_ENV
echo "INCLUDE=$INCLUDE;$INSTALL_PREFIX/include" >> $GITHUB_ENV
echo "OPENSSL_DIR=$WORKSPACE/OpenSSL" >> $GITHUB_ENV
echo "COMPILE_RUNTIME_DIR=$WORKSPACE/CompileRuntime" >> $GITHUB_ENV
echo "KIMAGEANNOTATOR_DIR=${INSTALL_PREFIX}" >> $GITHUB_ENV