/*
 * Copyright (C) 2021 Damir Porobic <damir.porobic@gmx.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "ImagePathStorage.h"

ImagePathStorage::ImagePathStorage() :
	mSettingsGroupPrefix(QLatin1String("recentImagesPath")),
	mSettingsGroupKey(QLatin1String("imagePath"))
{

}

void ImagePathStorage::store(const QString &value, int index)
{
	mSettings.beginWriteArray(mSettingsGroupPrefix);
	mSettings.setArrayIndex(index);
	mSettings.setValue(mSettingsGroupKey, value);
	mSettings.endArray();
	mSettings.sync();
}

QString ImagePathStorage::load(int index)
{
	mSettings.beginReadArray(mSettingsGroupPrefix);
	mSettings.setArrayIndex(index);
	auto value = mSettings.value(mSettingsGroupKey).toString();
	mSettings.endArray();
	return value;
}

int ImagePathStorage::count()
{
	auto count = mSettings.beginReadArray(mSettingsGroupPrefix);
	mSettings.endArray();
	return count;
}
